/***


Tetris Like : clone du célèbre jeu Tetris.


    Copyright (C) 2011  Frédéric Micout

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


***/

#ifdef __cplusplus
    #include <cstdlib>
#else
    #include <stdlib.h>
#endif

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_rotozoom.h>
#include <math.h>

#include <stdio.h>
#include <stdlib.h>

#define LARGEURGRILLE 192
#define LARGEURTOTALE 292
#define HAUTEURGRILLE 352
#define NBMOTIF 7


/*** Structure brique ***/
struct brique
{
    int couleur;
};


/*** Fonctions du programme ***/
bool testDeplacement(struct brique grilleJeu[LARGEURGRILLE/16][HAUTEURGRILLE/16],SDL_Rect offsetTMP[4])
{
    bool deplacementAutorise = true;
    int i;

    for(i=0;i<4;i++)
    {
        if (offsetTMP[i].x < 0) deplacementAutorise = false; // N'autorise pas le déplacement si le motif déborde de l'aire de jeu par la gauche
        if (offsetTMP[i].x > LARGEURGRILLE-16) deplacementAutorise = false; // N'autorise pas le déplacement si le motif déborde de l'aire de jeu par la droite
        if (offsetTMP[i].y > HAUTEURGRILLE-16) deplacementAutorise = false; // N'autorise pas le déplacement si le motif déborde de l'aire de jeu par le bas
        if (grilleJeu[offsetTMP[i].x/16][offsetTMP[i].y/16].couleur != 7) deplacementAutorise = false; // N'autorise pas le déplacement si le motif recouvre une case déjà occupée par une brique de couleur.
    }

    return deplacementAutorise;
}


/*** Boucle principale du programme ***/
int main ( int argc, char** argv )
{
    bool nouveauMotif = true; // Création d'un nouveau modif (vrai lorsqu'on lance le jeu)
    bool rotationMotif = false; // Rotation du motif
    bool descenteRapide = false; // Descente rapide du motif
    bool deplacementGauche = false;
    bool deplacementDroite = false;
    bool deplacementPossible = true;
    bool deplacementBasPossible = false;
    bool ligneComplete;
    bool pause = false;
    bool reset = false;
    bool finPartie;

    int i,j,k,digit;
    int typeMotif; // Type du motif en train de descendre
    int typeMotifSuivant = 7; // Type du prochain motif
    int angleMotif = 0; // Angle de rotation du motif
    int nbLignesDetruites = 0;
    int nbNouvellesLignesDetruites = 0;
    int nbPoints = 0;
    int vitesseJeu = 0;
    int compteurDescente = 0;
    int compteurDeplacementLateral = 0;
    int tempsActuel=0, tempsPrecedent=0;

    struct brique motifActuel[4];
    struct brique motifSuivant[4];

    // génération de la grille de jeu
    struct brique grilleJeu[LARGEURGRILLE/16][HAUTEURGRILLE/16];
    SDL_Rect offsetGrilleJeu[LARGEURGRILLE/16][HAUTEURGRILLE/16];

    // Création d'un nouvelle fenêtre
    SDL_Surface* fenetre = SDL_SetVideoMode(LARGEURTOTALE*2, HAUTEURGRILLE*2, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
	SDL_Surface* screen = SDL_CreateRGBSurface(SDL_HWSURFACE, LARGEURTOTALE, HAUTEURGRILLE, 32, 0, 0, 0, 0);
	SDL_Surface* screenX2 = NULL;

    // Initialisation de la grille de jeu
    for(i=0;i<LARGEURGRILLE/16;i++)
    {
        for(j=0;j<HAUTEURGRILLE/16;j++)
        {
            grilleJeu[i][j].couleur = 7;

            offsetGrilleJeu[i][j].x = 16*i;
            offsetGrilleJeu[i][j].y = 16*j;
            offsetGrilleJeu[i][j].h = 16;
            offsetGrilleJeu[i][j].w = 16;
        }
    }

    // Offsets attribués dans FENETRE
    SDL_Rect offsetBarreVerticale[HAUTEURGRILLE]; // Offset des barres verticales
    for (i=0;i<HAUTEURGRILLE;i++)
    {
        // Barre du milieu
        offsetBarreVerticale[i].x = LARGEURGRILLE;
        offsetBarreVerticale[i].y = 16*i;
    }

    SDL_Rect offset[4]; // Offset du motif en cours de descente
    SDL_Rect offsetTMP[4]; // Offset temporaire utilisé pour tester les collisions à venir

    // Offset du motif suivant
    SDL_Rect offsetSuivant[4];
    for (i=0;i<4;i++)
    {
        offsetSuivant[i].x = 0;
        offsetSuivant[i].y = HAUTEURGRILLE;
        motifSuivant[i].couleur = 0;

    }

    // Pour le découpage de l'image contenant les images des briques
    SDL_Rect imageBriques[9];
    for (i=0;i<9;i++)
    {
        imageBriques[i].x = i*16;
        imageBriques[i].y = 0;
        imageBriques[i].h = 16;
        imageBriques[i].w = 16;
    }

    // Pour le positionnement du score
    SDL_Rect offsetDigitScore[5];
    for (i=0;i<5;i++)
    {
        offsetDigitScore[i].x = LARGEURGRILLE + 32 + 8*i;
        offsetDigitScore[i].y = 120;
        offsetDigitScore[i].h = 8;
        offsetDigitScore[i].w = 8;
    }

    SDL_Rect offsetDigitLignes[3];
    for (i=0;i<3;i++)
    {
        offsetDigitLignes[i].x = LARGEURGRILLE + 48 + 8*i;
        offsetDigitLignes[i].y = 140;
        offsetDigitLignes[i].h = 8;
        offsetDigitLignes[i].w = 8;
    }

    // Pour le découpage de l'image contenant les caractères alphanumériques
    SDL_Rect imageAlphaNum[13][3];
    for (i=0;i<13;i++)
    {
        for (j=0;j<3;j++)
        {
            imageAlphaNum[i][j].x = i*8;
            imageAlphaNum[i][j].y = j*8;
            imageAlphaNum[i][j].h = 8;
            imageAlphaNum[i][j].w = 8;
        }
    }



    // Chargement des images
    SDL_Surface* pBriques = IMG_Load("briques.png");
    SDL_Surface* pAlphaNum = IMG_Load("alphaNum.png");





    // initialisation de la SDL
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        printf( "Impossible d'initialiser la SDL : %s\n", SDL_GetError() );
        return 1;
    }

    // Pour ne pas aficher le pointer de la souris
    SDL_ShowCursor(SDL_DISABLE);

    // Pour une sortie propre de la SDL
    atexit(SDL_Quit);


    // Boucle principale du programme
    bool done = false;
    while (!done)
    {

        // Gestion des évènements
        SDL_Event event;
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_QUIT:
                    done = true;
                    break;

                case SDL_KEYDOWN:
                    switch(event.key.keysym.sym)
                    {
                        case SDLK_ESCAPE:
                            done = true;
                            break;

                        // Touches de direction
                        case SDLK_UP:
                            if(!pause)
                            {
                                if(!rotationMotif) angleMotif++;
                                rotationMotif = true;
                            }
                            break;

                        case SDLK_DOWN:
                            descenteRapide = true;
                            break;

                        case SDLK_LEFT:
                            deplacementGauche = true;
                            deplacementDroite = false;
                            compteurDeplacementLateral = 0;
                            break;

                        case SDLK_RIGHT:
                            deplacementGauche = false;
                            deplacementDroite = true;
                            compteurDeplacementLateral = 0;
                            break;

                        case SDLK_p:
                            if(!pause) pause = true;
                            else pause = false;
                            break;

                        case SDLK_r:
                            reset = true;
                            break;

                        // Cas pour les autres touches
                        default:
                            break;
                    }
                    break;

                case SDL_KEYUP:
                    switch(event.key.keysym.sym)
                    {
                        case SDLK_DOWN:
                             descenteRapide = false;
                            break;

                        case SDLK_LEFT:
                            deplacementGauche = false;
                            deplacementDroite = false;
                            break;

                        case SDLK_RIGHT:
                            deplacementGauche = false;
                            deplacementDroite = false;
                            break;

                        // Cas pour les autres touches
                        default:
                            break;
                    }
                    break;

                default:
                    break;
            }
        }



        // Appel sous condition à la fonction de création d'un nouveau motif
        if (nouveauMotif)
        {


            typeMotif = typeMotifSuivant;
            for(i=0;i<4;i++)
            {
                motifActuel[i].couleur = typeMotif;
                offset[i].x = offsetSuivant[i].x - 140;
                offset[i].y = offsetSuivant[i].y - 20;
            }

            // Création d'un nouveau motif de manière aléatoire (parmis les 7 possibles)
            typeMotifSuivant = (rand()* SDL_GetTicks() )%NBMOTIF;

            // Initialisation des autres variables du motif
            deplacementBasPossible = true;
            descenteRapide = false;
            compteurDeplacementLateral = 0;
            angleMotif = 0;
            compteurDescente = 0;

            switch (typeMotifSuivant)
            {
                case 0:
                    for(i=0;i<4;i++) // Barre
                    {
                        offsetSuivant[i].x = LARGEURGRILLE/2;
                        offsetSuivant[i].y = 16 * i;
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 1:
                    for(i=0;i<4;i++) // Carré
                    {
                        offsetSuivant[i].x = LARGEURGRILLE/2 + (i/2) * 16;
                        offsetSuivant[i].y = 16 * (i%2);
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 2: // Pyramide
                    offsetSuivant[0].x = LARGEURGRILLE/2;
                    offsetSuivant[0].y = 0;
                    offsetSuivant[1].x = LARGEURGRILLE/2 - 16;
                    offsetSuivant[1].y = 16;
                    offsetSuivant[2].x = LARGEURGRILLE/2;
                    offsetSuivant[2].y = 16;
                    offsetSuivant[3].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[3].y = 16;
                    for(i=0;i<4;i++)
                    {
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 3: // Anti-slash
                    offsetSuivant[0].x = LARGEURGRILLE/2;
                    offsetSuivant[0].y = 0;
                    offsetSuivant[1].x = LARGEURGRILLE/2;
                    offsetSuivant[1].y = 16;
                    offsetSuivant[2].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[2].y = 16;
                    offsetSuivant[3].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[3].y = 32;
                    for(i=0;i<4;i++)
                    {
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 4: // Slash
                    offsetSuivant[0].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[0].y = 0;
                    offsetSuivant[1].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[1].y = 16;
                    offsetSuivant[2].x = LARGEURGRILLE/2;
                    offsetSuivant[2].y = 16;
                    offsetSuivant[3].x = LARGEURGRILLE/2;
                    offsetSuivant[3].y = 32;
                    for(i=0;i<4;i++)
                    {
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 5: // L
                    offsetSuivant[0].x = LARGEURGRILLE/2;
                    offsetSuivant[0].y = 0;
                    offsetSuivant[1].x = LARGEURGRILLE/2;
                    offsetSuivant[1].y = 16;
                    offsetSuivant[2].x = LARGEURGRILLE/2;
                    offsetSuivant[2].y = 32;
                    offsetSuivant[3].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[3].y = 32;
                    for(i=0;i<4;i++)
                    {
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                case 6: // J
                    offsetSuivant[0].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[0].y = 0;
                    offsetSuivant[1].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[1].y = 16;
                    offsetSuivant[2].x = LARGEURGRILLE/2 + 16;
                    offsetSuivant[2].y = 32;
                    offsetSuivant[3].x = LARGEURGRILLE/2;
                    offsetSuivant[3].y = 32;
                    for(i=0;i<4;i++)
                    {
                        motifSuivant[i].couleur = typeMotifSuivant;
                    }
                    break;

                default:
                    break;
            }
            // Plus de nouveau motif à créer pour le moment
            nouveauMotif = false;

            for(i=0;i<4;i++)
            {
                offsetSuivant[i].x += 140;
                offsetSuivant[i].y += 20;
            }
        }



        if(!pause)
        {

            // Déplacement du motif vers la gauche demandé
            if(deplacementGauche)
            {
                for(i=0;i<4;i++)
                {
                    offsetTMP[i].x = offset[i].x - 16;
                    offsetTMP[i].y = offset[i].y;
                }
                // Test si déplacement gauche possible
                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);


                // Déplacement gauche validé
                if(deplacementPossible && (compteurDeplacementLateral == 0 || compteurDeplacementLateral > 3))
                {
                    for(i=0;i<4;i++)
                    {
                        offset[i] = offsetTMP[i];
                    }
                }
            }

            // Déplacement du motif vers la droite demandé
            if(deplacementDroite)
            {
                for(i=0;i<4;i++)
                {
                    offsetTMP[i].x = offset[i].x + 16;
                    offsetTMP[i].y = offset[i].y;
                }
                // Test si déplacement gauche possible
                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);


                // Déplacement gauche validé
                //if(deplacementPossible && compteurDeplacementLateral%2 == 0)
                if(deplacementPossible && (compteurDeplacementLateral == 0 || compteurDeplacementLateral > 3))
                {
                    for(i=0;i<4;i++)
                    {
                        offset[i] = offsetTMP[i];
                    }
                }
            }

            // Rotation du motif demandé
            if(rotationMotif)
            {
                // Copie de l'offset du motif dans un tableau temporaire
                for (i=0;i<4;i++) offsetTMP[i] = offset[i];

                // tenter d'effectuer la rotation du motif (en fonction du typeMotif)
                switch (typeMotif)
                {
                    case 0: // Barre
                        switch (angleMotif%2)
                        {
                            case 0:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y -= 16;

                                offsetTMP[2].x += 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].x += 32;
                                offsetTMP[3].y += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 1:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y += 16;

                                offsetTMP[2].x -= 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].x -= 32;
                                offsetTMP[3].y -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            default:
                                break;
                        }
                        break;

                    case 1: // Carré = rien à faire
                        break;

                    case 2: // Pyramide
                        switch (angleMotif%4)
                        {
                            case 0:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[1].x -= 16;
                                offsetTMP[1].y += 16;
                                offsetTMP[3].x += 16;
                                offsetTMP[3].y -= 16;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 3;

                                break;

                            case 1:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[1].x += 16;
                                offsetTMP[1].y += 16;
                                offsetTMP[3].x -= 16;
                                offsetTMP[3].y -= 16;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            case 2:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[1].x += 16;
                                offsetTMP[1].y -= 16;
                                offsetTMP[3].x -= 16;
                                offsetTMP[3].y += 16;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 3:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[1].x -= 16;
                                offsetTMP[1].y -= 16;
                                offsetTMP[3].x += 16;
                                offsetTMP[3].y += 16;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 2;

                                break;

                            default:
                                break;
                        }
                        break;

                    case 3: // Anti-slash
                        switch (angleMotif%2)
                        {
                            case 0:

                                offsetTMP[0].y -= 16;
                                offsetTMP[1].x -= 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].x -= 16;
                                offsetTMP[3].y += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 1:

                                offsetTMP[0].y += 16;
                                offsetTMP[1].x += 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].x += 16;
                                offsetTMP[3].y -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            default:
                                break;
                        }
                        break;

                    case 4: // Slash
                        switch (angleMotif%2)
                        {
                            case 0:

                                offsetTMP[0].y -= 16;
                                offsetTMP[1].x += 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].x += 16;
                                offsetTMP[3].y += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 1:

                                offsetTMP[0].y += 16;
                                offsetTMP[1].x -= 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].x -= 16;
                                offsetTMP[3].y -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            default:
                                break;
                        }
                        break;

                    case 5: // L
                        switch (angleMotif%4)
                        {
                            case 0:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[2].x += 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].x += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 3;

                                break;

                            case 1:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[2].x += 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].y -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            case 2:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[2].x -= 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].x -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 3:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[2].x -= 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].y += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 2;

                                break;

                            default:
                                break;
                        }
                        break;

                    case 6: // J
                        switch (angleMotif%4)
                        {
                            case 0:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[2].x += 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].y += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 3;

                                break;

                            case 1:

                                offsetTMP[0].x -= 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[2].x += 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].x += 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 0;

                                break;

                            case 2:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y += 16;
                                offsetTMP[2].x -= 16;
                                offsetTMP[2].y -= 16;
                                offsetTMP[3].y -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 1;

                                break;

                            case 3:

                                offsetTMP[0].x += 16;
                                offsetTMP[0].y -= 16;
                                offsetTMP[2].x -= 16;
                                offsetTMP[2].y += 16;
                                offsetTMP[3].x -= 32;

                                deplacementPossible = testDeplacement(grilleJeu,offsetTMP);

                                if(deplacementPossible) for (i=0;i<4;i++) offset[i] = offsetTMP[i];
                                else angleMotif = 2;

                                break;

                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

            rotationMotif = false;
            }

            // Déplacement vers le bas demandé
            for(i=0;i<4;i++)
            {
                offsetTMP[i].x = offset[i].x;
                offsetTMP[i].y = offset[i].y + 16;
            }

            // Test si déplacement bas possible
            deplacementBasPossible = testDeplacement(grilleJeu,offsetTMP);

            // Déplacement vers le bas
            for(i=0;i<4;i++)
            {
                if (!deplacementBasPossible) offset[i].y += 0;
                else if(descenteRapide) offset[i].y += 16;
                else if(compteurDescente%(30-vitesseJeu/3) == 0) offset[i].y += 16;
            }

        }


        // Si le déplacement vers le bas n'est plus possible
        if (!deplacementBasPossible && (compteurDescente%(30-vitesseJeu/3) == 29-vitesseJeu/3 || compteurDescente%(30-vitesseJeu/3) == 12 || !deplacementPossible))
        {
            // Ajouter les briques du motif à la pile et à la même place
            for(i=0;i<4;i++) grilleJeu[offset[i].x/16][offset[i].y/16].couleur = motifActuel[i].couleur;

            // Demander la création d'un nouveau motif
            nouveauMotif = true;
        }

        // Initialisation du nombre d lignes détruites dans ce tour de boucle
        nbNouvellesLignesDetruites = 0;

        // Suppression de ligne s'il y en a de complètes
        for(j=0;j<HAUTEURGRILLE/16;j++)
        {
            ligneComplete = true;
            for(i=0;i<LARGEURGRILLE/16;i++) if(grilleJeu[i][j].couleur == 7) ligneComplete = false; // Test si la ligne est complète

            if(ligneComplete)
            {
                for(i=0;i<LARGEURGRILLE/16;i++) grilleJeu[i][j].couleur = 7; // La ligne complète est effacée

                // Descente de toutes les briques du dessus d'un cran vers le bas
                for(k=j;k>1;k--)
                {
                    for(i=0;i<LARGEURGRILLE/16;i++) grilleJeu[i][k].couleur = grilleJeu[i][k-1].couleur;
                }

                nbNouvellesLignesDetruites++;
            }

        }

        nbLignesDetruites += nbNouvellesLignesDetruites;

        compteurDescente++;
        compteurDeplacementLateral++;

        // Mise à jour du score
        switch(nbNouvellesLignesDetruites)
        {
            case 1:
                nbPoints += 10;
                break;

            case 2:
                nbPoints += 25;
                break;

            case 3:
                nbPoints += 50;
                break;

            case 4:
                nbPoints += 100;
                break;

            default:
                break;

        }

        // Evolution de la vitesse de descente des motifs
        if(nbNouvellesLignesDetruites != 0)
        {
            vitesseJeu += nbNouvellesLignesDetruites;
            if(vitesseJeu > 60) vitesseJeu = 60;
        }


        // Reset partie
        if(reset)
        {
            // initialisation de la grille
            for(i=0;i<LARGEURGRILLE/16;i++)
            {
                for(j=0;j<HAUTEURGRILLE/16;j++)
                {
                    grilleJeu[i][j].couleur = 7;

                    offsetGrilleJeu[i][j].x = 16*i;
                    offsetGrilleJeu[i][j].y = 16*j;
                    offsetGrilleJeu[i][j].h = 16;
                    offsetGrilleJeu[i][j].w = 16;
                }
            }

            // initialisation des compteurs
            vitesseJeu = 0;
            nbPoints = 0;
            nbLignesDetruites = 0;

            nouveauMotif = true;
            reset = false;

        }

        // Fin de partie
        for(i=0;i<LARGEURGRILLE/16;i++)
        {
            if(grilleJeu[i][0].couleur != 7)
            {
                finPartie = true;
                nouveauMotif = false;
            }
        }







        // Effacement écran précédent
        SDL_FillRect(screen, 0, SDL_MapRGB(screen->format, 0, 0, 0));

        // Affichage des briques figées
        for(i=0;i<LARGEURGRILLE/16;i++)
        {
            for(j=0;j<HAUTEURGRILLE/16;j++)
            {
                SDL_BlitSurface(pBriques, &imageBriques[grilleJeu[i][j].couleur], screen, &offsetGrilleJeu[i][j] );
            }
        }

        // Affichage de la barre verticale de séparation
        for(i=0;i<HAUTEURGRILLE/16;i++) SDL_BlitSurface(pBriques, &imageBriques[8], screen, &offsetBarreVerticale[i] );

        // Affichage du motif en cours de descente
        for(i=0;i<4;i++) SDL_BlitSurface(pBriques, &imageBriques[typeMotif], screen, &offset[i] );

        // Affichage du prochain motif
        for(i=0;i<4;i++) SDL_BlitSurface(pBriques, &imageBriques[typeMotifSuivant], screen, &offsetSuivant[i] );

        // Affichage du score (5 digits)
        for(i=0;i<5;i++)
        {
            j = (nbPoints/(pow(10.0,(4-i))));
            digit = j%10;
            if(nbPoints>=(pow(10.0,4-i))) SDL_BlitSurface(pAlphaNum, &imageAlphaNum[digit][0], screen, &offsetDigitScore[i] );
            else if (i==4) SDL_BlitSurface(pAlphaNum, &imageAlphaNum[0][0], screen, &offsetDigitScore[i] );
        }

        // Affichage du nombre de lignes détruites (3 digits)
        for(i=0;i<3;i++)
        {
            j = (nbLignesDetruites/(pow(10.0,(2-i))));
            digit = j%10;
            if(nbLignesDetruites>=(pow(10.0,2-i))) SDL_BlitSurface(pAlphaNum, &imageAlphaNum[digit][0], screen, &offsetDigitLignes[i] );
            else if (i==2) SDL_BlitSurface(pAlphaNum, &imageAlphaNum[0][0], screen, &offsetDigitLignes[i] );
        }

        // Modifie la taille de l'écran
        SDL_FreeSurface(screenX2);
        screenX2 = zoomSurface (screen, 2, 2, 0);
        SDL_BlitSurface(screenX2, NULL, fenetre, NULL);

        // Rafraichissement de l'affichage
        SDL_Flip(fenetre);

        // Gestion des fps
        do
        {
            tempsActuel = SDL_GetTicks();
            SDL_Delay(1);
        }
        while(tempsActuel<tempsPrecedent+33);
        tempsPrecedent = tempsActuel;

    }
    // Pour quitter proprement le programme
	SDL_FreeSurface(screen);
	SDL_FreeSurface(screenX2);
	SDL_FreeSurface(fenetre);

    SDL_ShowCursor(SDL_ENABLE);

	SDL_Quit();

    return 0;
}
